/*-------------------------------------------------------------------------------------------------------------------*\
|  Copyright (C) 2014 eBay Software Foundation                                                                        |
|                                                                                                                     |
|  Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance     |
|  with the License.                                                                                                  |
|                                                                                                                     |
|  You may obtain a copy of the License at                                                                            |
|                                                                                                                     |
|       http://www.apache.org/licenses/LICENSE-2.0                                                                    |
|                                                                                                                     |
|  Unless required by applicable law or agreed to in writing, software distributed under the License is distributed   |
|  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for  |
|  the specific language governing permissions and limitations under the License.                                     |
\*-------------------------------------------------------------------------------------------------------------------*/

package com.paypal.selion.reports.reporter.excelreport;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Level;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.testng.IReporter;
import org.testng.IResultMap;
import org.testng.ISuite;
import org.testng.ISuiteResult;
import org.testng.ITestContext;
import org.testng.ITestNGMethod;
import org.testng.ITestResult;
import org.testng.xml.XmlSuite;

import com.paypal.selion.logger.SeLionLogger;
import com.paypal.selion.reports.reporter.html.ReportDataGenerator;
import com.paypal.selion.reports.reporter.services.ConfigSummaryData;
import com.paypal.test.utilities.logging.SimpleLogger;

/**
 * 
 * <br>
 * Add this class as listener to generate an ExcelReport for a suite run after the SoftAssertCapabilities file. </br>
 * Implements the IReporter interface to fetch data from TestNG. Depends on BFReporter to generate links based data
 */
@SuppressWarnings("unchecked")
public class ExcelReport implements IReporter {
    private static SimpleLogger logger = SeLionLogger.getLogger();

    private HSSFWorkbook wb;

    private List<SummarizedData> lSuites = new ArrayList<SummarizedData>();
    private List<SummarizedData> lTests = new ArrayList<SummarizedData>();
    private List<SummarizedData> lGroups = new ArrayList<SummarizedData>();
    private List<SummarizedData> lClasses = new ArrayList<SummarizedData>();
    private ArrayList<TestCaseResult> allTestsResults = new ArrayList<TestCaseResult>();

    private List<List<String>> tcFailedData = new ArrayList<List<String>>();
    private List<List<String>> tcPassedData = new ArrayList<List<String>>();
    private List<List<String>> tcSkippedData = new ArrayList<List<String>>();
    private List<List<String>> tcDefectData = new ArrayList<List<String>>();

    private Map<String, SummarizedData> mpGroupClassData = new HashMap<String, SummarizedData>();

    private static List<ReportMap<?>> fullReportMap = new ArrayList<ReportMap<?>>();

    /**
     * The first method that gets called when generating the report. Generates data in way the Excel should appear.
     * Creates the Excel Report and writes it to a file.
     */
    public void generateReport(List<XmlSuite> xmlSuites, List<ISuite> suites, String sOpDirectory) {
        logger.entering(new Object[] { xmlSuites, suites, sOpDirectory });
        if (logger.isLoggable(Level.INFO)) {
            logger.log(Level.INFO, "Generating ExcelReport");
        }

        ReportDataGenerator.initReportData(suites);
        TestCaseResult.setOutputDirectory(sOpDirectory);
        // Generate data to suit excel report.
        this.generateSummaryData(suites);
        this.generateTCBasedData(allTestsResults);

        // Create the Excel Report
        this.createExcelReport();

        // Render the report
        File outputFile = new File(sOpDirectory + "/Excel_Report.xls");
        try {
            FileOutputStream fOut = new FileOutputStream(outputFile);
            wb.write(fOut);
            fOut.flush();
        } catch (FileNotFoundException e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
        } catch (IOException e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
        }
        if (logger.isLoggable(Level.INFO)) {
            logger.log(Level.INFO, "Excel File Created @ " + outputFile.getAbsolutePath());
        }
    }

    /**
     * Initialized styles used in the workbook. Generates the report related info. Creates the structure of the Excel
     * Reports
     */
    private void createExcelReport() {
        logger.entering();

        wb = new HSSFWorkbook();
        Styles.initStyles(wb);

        // Report Details
        this.createReportInfo();

        // Map of sheet names - individual reports and corresponding data
        this.createReportMap();

        // Render reports in the Workbook
        for (ReportMap rm : fullReportMap) {
            List<BaseReport<?>> allReports = rm.getGeneratedReport();
            allReports.iterator().next().generateRep(this.wb, rm.getName(), rm.getGeneratedReport());
        }

        logger.exiting();
    }

    /**
     * Create Run details like owner of run, time and stage used.
     */
    private void createReportInfo() {
        logger.entering();

        HSSFSheet summarySheet = wb.createSheet(ReportSheetNames.TESTSUMMARYREPORT.getName());

        Map<String, String> reportInfo = new LinkedHashMap<String, String>();

        for (Entry<String, String> temp : ConfigSummaryData.getConfigSummary().entrySet()) {
            reportInfo.put(temp.getKey(), temp.getValue());
        }

        int rowNum = 0, colNum = 0;
        HSSFCell col;
        HSSFRow row;
        for (Entry<String, String> eachReportInfo : reportInfo.entrySet()) {
            colNum = 2;
            row = summarySheet.createRow(rowNum++);

            col = row.createCell(colNum);
            col.setCellStyle(Styles.getSubHeading2Style());
            col.setCellValue(eachReportInfo.getKey());

            // Next column holds the values
            col = row.createCell(++colNum);
            col.setCellStyle(Styles.getThinBorderStyle());
            col.setCellValue(eachReportInfo.getValue());
        }

        logger.exiting();
    }

    /**
     * Creates all the report details like which sheet should contain which report and the data associated with the
     * report
     */
    private void createReportMap() {
        logger.entering();

        // Summary Report
        Map<String, List<SummarizedData>> subReportMap = new LinkedHashMap<String, List<SummarizedData>>();
        subReportMap.put("Full Suite Summary", lSuites);
        subReportMap.put("Test Summary", lTests);
        subReportMap.put("Classwise Summary", lClasses);
        subReportMap.put("Groupwise Summary", lGroups);

        ReportMap<SummarizedData> testSummaryReport = new ReportMap<SummarizedData>(
                ReportSheetNames.TESTSUMMARYREPORT.getName(), subReportMap, 0);
        fullReportMap.add(testSummaryReport);

        // Group Detailed Report
        List<SummarizedData> groupsClone = new ArrayList<SummarizedData>(lGroups);
        List<SummarizedData> classData;

        SummarizedData naGroupData = new SummarizedData();
        naGroupData.setsName(TestCaseResult.NA);
        groupsClone.add(naGroupData);
        subReportMap = new LinkedHashMap<String, List<SummarizedData>>();
        for (SummarizedData group : groupsClone) {

            String sGroupName = group.getsName();
            classData = new ArrayList<SummarizedData>();
            for (String sGroupClassName : mpGroupClassData.keySet()) {
                if (sGroupClassName.substring(0, sGroupName.length()).equals(sGroupName)) {
                    classData.add(mpGroupClassData.get(sGroupClassName));
                }
            }
            subReportMap.put(sGroupName, classData);
        }

        ReportMap<SummarizedData> secondReport = new ReportMap<SummarizedData>(
                ReportSheetNames.GROUPSUMMARYREPORT.getName(), subReportMap, 0);
        fullReportMap.add(secondReport);

        // TestCase Status Report
        Map<String, List<List<String>>> subDetailReportMap = new LinkedHashMap<String, List<List<String>>>();
        subDetailReportMap.put("Passed TC List", tcPassedData);
        subDetailReportMap.put("Failed TC List", tcFailedData);
        subDetailReportMap.put("Skipped TC List", tcSkippedData);

        ReportMap<List<String>> thirdReport = new ReportMap<List<String>>(ReportSheetNames.TESTCASEREPORT.getName(),
                subDetailReportMap, 1);
        fullReportMap.add(thirdReport);

        // Defect Report
        Map<String, List<List<String>>> lstDefectReports = new LinkedHashMap<String, List<List<String>>>();
        lstDefectReports.put("Defect Summary", tcDefectData);
        ReportMap<List<String>> fourthReport = new ReportMap<List<String>>(ReportSheetNames.DEFECTREPORT.getName(),
                lstDefectReports, 1);
        fullReportMap.add(fourthReport);

        // Changing the titles of the Defect Report
        BaseReport<List<String>> bR = (BaseReport<List<String>>) fullReportMap.get(fullReportMap.size() - 1)
                .getGeneratedReport().iterator().next();
        List<String> lsTitles = Arrays.asList(new String[] { "Class Name", "Method/Testcase id", "Test Description",
                "Group[s]", "Time taken", "Link", "Error Message", "Error Details" });
        bR.setColTitles(lsTitles);
        logger.exiting();
    }

    /**
     * Generates all summarized counts for various reports
     * 
     * @param suites
     */
    private void generateSummaryData(List<ISuite> suites) {
        logger.entering(suites);

        SummarizedData tempSuite = null;
        SummarizedData tempTest;
        SummarizedData tempGroups = null;
        this.generateTestCaseResultData(suites);

        // Generating Group Summary data
        for (ISuite suite : suites) {

            tempSuite = new SummarizedData();
            tempSuite.setsName(suite.getName());
            Map<String, ISuiteResult> allResults = suite.getResults();
            for (Entry<String, Collection<ITestNGMethod>> sGroupName : suite.getMethodsByGroups().entrySet()) {
                tempGroups = new SummarizedData();
                tempGroups.setsName(sGroupName.getKey());
                tempGroups.incrementiTotal(sGroupName.getValue().size());

                for (TestCaseResult tr : allTestsResults) {
                    if (tr.getGroup().contains(sGroupName.getKey())) {
                        tempGroups.incrementCount(tr.getStatus());
                        tempGroups.incrementDuration(tr.getDurationTaken());
                    }
                }
                tempGroups.setiTotal(tempGroups.getiPassedCount() + tempGroups.getiFailedCount()
                        + tempGroups.getiSkippedCount());
                lGroups.add(tempGroups);

            }

            // Generating Test summary data
            for (ISuiteResult testResult : allResults.values()) {
                ITestContext testContext = testResult.getTestContext();
                tempTest = new SummarizedData();
                tempTest.setsName(testContext.getName());
                tempTest.setiFailedCount(testContext.getFailedTests().size());
                tempTest.setiPassedCount(testContext.getPassedTests().size());
                tempTest.setiSkippedCount(testContext.getSkippedTests().size());
                tempTest.setiTotal(tempTest.getiPassedCount() + tempTest.getiFailedCount()
                        + tempTest.getiSkippedCount());
                tempTest.setlRuntime(testContext.getEndDate().getTime() - testContext.getStartDate().getTime());

                lTests.add(tempTest);
            }

            // Generating Suite Summary data
            for (SummarizedData test : lTests) {

                tempSuite.setiPassedCount(test.getiPassedCount() + tempSuite.getiPassedCount());
                tempSuite.setiFailedCount(test.getiFailedCount() + tempSuite.getiFailedCount());
                tempSuite.setiSkippedCount(tempSuite.getiSkippedCount() + test.getiSkippedCount());
                tempSuite.setiTotal(tempSuite.getiPassedCount() + tempSuite.getiFailedCount()
                        + tempSuite.getiSkippedCount());
                tempSuite.setlRuntime(test.getlRuntime() + tempSuite.getlRuntime());
            }
            lSuites.add(tempSuite);

        }

        Collections.sort(lGroups);
        Collections.sort(lTests);
        logger.exiting();
    }

    /**
     * Method to generate array of all results of all testcases that were run in a suite Output : Populates the
     * allTestsResults arraylist with results and info for all test methods.
     */
    private void generateTestCaseResultData(List<ISuite> suites) {
        logger.entering();
        for (ISuite suite : suites) {

            Map<String, ISuiteResult> allResults = suite.getResults();

            for (ISuiteResult testResult : allResults.values()) {

                ITestContext testContext = testResult.getTestContext();

                IResultMap passedResultMap = testContext.getPassedTests();
                IResultMap failedResultMap = testContext.getFailedTests();
                IResultMap skippedResultMap = testContext.getSkippedTests();

                this.allTestsResults.addAll(this.createResultFromMap(passedResultMap));
                this.allTestsResults.addAll(this.createResultFromMap(failedResultMap));
                this.allTestsResults.addAll(this.createResultFromMap(skippedResultMap));
            }
        }
        logger.exiting();
    }

    /**
     * Generates individual TestCase Results based on map of passed, failed and skipped methods Returns the list of
     * TestCaseResult objects generated.
     */
    private List<TestCaseResult> createResultFromMap(IResultMap resultMap) {
        logger.entering(resultMap);
        List<TestCaseResult> statusWiseResults = new ArrayList<TestCaseResult>();

        for (ITestResult singleMethodResult : resultMap.getAllResults()) {
            TestCaseResult tcresult1 = new TestCaseResult();
            tcresult1.setITestResultobj(singleMethodResult);
            statusWiseResults.add(tcresult1);

        }
        Collections.sort(statusWiseResults);
        logger.exiting(statusWiseResults);
        return statusWiseResults;
    }

    /**
     * Generates class based summary and the basis for Detailed groupwise summary report
     */
    private void generateTCBasedData(ArrayList<TestCaseResult> allTestsList) {
        logger.entering(allTestsList);
        SummarizedData tempClass = null, tempGroupClass = null;
        Map<String, SummarizedData> mpClassData = new HashMap<String, SummarizedData>();

        for (TestCaseResult tcResult : allTestsList) {

            // Segregating for class data
            String sTempClassName = tcResult.getClassName();

            // If class not already added to Class data, then create new ClassObject exists
            if (!mpClassData.containsKey(sTempClassName)) {
                tempClass = new SummarizedData();
                tempClass.setsName(sTempClassName);

            } else {
                tempClass = mpClassData.get(sTempClassName);
            }

            // Adding test to total count
            tempClass.incrementiTotal();

            // Adding all groups to map
            for (String sGroup : tcResult.getGroup()) {

                // Forming a key for the GroupClass map which is <GroupName><ClassName>
                String sGroupClassName = sGroup + sTempClassName;
                if (!mpGroupClassData.containsKey(sGroupClassName)) {
                    tempGroupClass = new SummarizedData();
                    tempGroupClass.setsName(sTempClassName);
                } else {
                    tempGroupClass = mpGroupClassData.get(sGroupClassName);
                }

                tempGroupClass.incrementiTotal();
                tempGroupClass.incrementCount(tcResult.getStatus());
                tempGroupClass.incrementDuration(tcResult.getDurationTaken());
                mpGroupClassData.put(sGroupClassName, tempGroupClass);
            }

            // Segregating for detailed Testcase Status wise data
            List<String> str = new ArrayList<String>();
            str.add(tcResult.getClassName());
            str.add(tcResult.getMethodName());
            str.add(tcResult.getTestDesc());
            str.add(tcResult.getGroup().toString());
            str.add(String.valueOf(tcResult.getDurationTaken()));
            str.add(tcResult.getsslink());

            // Based on status, incrementing class count and adding str to correct
            // list for TC detailed report
            switch (tcResult.getStatus()) {
            case ITestResult.FAILURE: {
                tcFailedData.add(str);
                // For failed cases adding data for defect description sheet
                for (int iErrorCount = 0; iErrorCount < tcResult.getError().size(); iErrorCount++) {
                    List<String> tmpList = new ArrayList<String>();
                    tmpList.addAll(0, str);
                    tmpList.add(tcResult.getDefect().get(iErrorCount));
                    tmpList.add(tcResult.getError().get(iErrorCount));
                    tcDefectData.add(tmpList);
                }
                break;
            }
            case ITestResult.SUCCESS: {
                tcPassedData.add(str);
                break;
            }
            case ITestResult.SKIP: {
                tcSkippedData.add(str);
                break;
            }
            }
            tempClass.incrementCount(tcResult.getStatus());
            // Add to the total runtime of the class
            tempClass.setlRuntime(tempClass.getlRuntime() + tcResult.getDurationTaken());
            mpClassData.put(sTempClassName, tempClass);
        }
        SummarizedData[] ps = new SummarizedData[mpClassData.size()];
        lClasses = Arrays.asList(mpClassData.values().toArray(ps));
        Collections.sort(lClasses);
        logger.exiting();
    }

}
